/*
    dds2tar  Digital-Data-Storage Extract Tool

             This tool makes use of the fast seek command of DAT devices.
             Files from a selected file archive can be extracted withing
             one minute.

             J"org Weule                     weule@cs.uni-duesseldorf.de
                                             Fon: +49 211 751409

----------------------------------------------------------------------------

                              LICENCE

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published
    by the Free Software Foundation; either version 1, or (at your option) 
    any	later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

----------------------------------------------------------------------------

    To anyone in germany I decalre the following:

          Ich werde gegen jede Verbreitung der Software rechtlich
          vorgehen, wenn dabei die GNU General Public Lisence (GPL) nicht
          eingehalten wird.

          Schon mit dem Kopieren oder Benutzen von Teilen dieser Software
          wird die GPL stillschweigend akzeptiert. Lesen Sie also die 
          Lizensbedinungen sorgf"altig! Unkenntnis sch"utzt nicht vor Strafe.

*/

/* changes
    1.1.2    added an option to change the compression mode of the device
    1.1.0    added the options -R, -b, -e to support usage of the tar
                record listing (tar -tvR)
             removed the option -d and removed the option ON_THE_FLY
                this options cased troubles
    1.0.2    added the define ON_THE_FLY
    1.0.1    added the define FIXED_SIZE
 */

#include <stdlib.h>
#include <stdio.h>
#include <sys/mtio.h>
#include <sys/types.h>
#include <fcntl.h>
#include <unistd.h>
#include <string.h>
#include "dds2tar.h"

static char*help_text = 
"usage: %s [options... ]                storing the index                \n"
"usage: %s [options... ] -p prefix-list using the index, extracting files\n"
"usage: %s [options... ] -x file-list   using the index, extracting files\n"
"usage: %s [options... ] -g pat-list    using the index, extracting files\n"
"  -t F   selecting index file                  \n"
"  -R F   selecting 'tar -R' index file         \n" /* record_file */
"  -e F   selecting environment file            \n"
"  -f F   selecting device                      \n"
"  -z     using index-file at compressed format \n"
"  -b     set block size                        \n"
"  -L     print location                        \n"
"  -v     verbose                               \n"
"  --hash-mode                                  \n"
"                                               \n"
"  ........ to change the compression mode of the device ................\n"
"usage: %s [-f device] <--ddscm-on|--ddscm-off|--ddscm-query|--ddscm-log> \n"
"                                               \n"
;

static char*Alpha="\n\
usage: %s [options... ] -l location-list    extracting files at locations\n\
";

    static char*version="Version 1.1.0 -- last change 1994 May 10";
/*  static char*version="Version 1.0.2 -- last change 1994 May 2";    */
/*  static char*version="Version 1.0.1 -- last change 1994 April 30"; */
/*  static char*version="Version 1.0 -- last change 1994 March 6"; */

/*-------------------------------------------------------------------------*/

/* --- flags --- */
int mode           = 0 ;
int verbose        = 0 ;
int all            = 0 ;
int dump_mode      = 0 ;
int hash_mode      = 0 ;
int print_location = 0 ;
int tar_index_file = 0 ;
int device_change  = 0 ; /* change the mopde of the device */
int device_comp    = 0 ; /* turn compression on/off */

int device ;
char*device_name=NULL;

/* file io */
FILE*index_fp      = 0 ;
char*index_file    = NULL ;

/* archive location */
int tar_fb  = 0 ;		/* first block of the archive */
int tar_bs  = 20 ;		/* block size in records of 512 bytes */
int tar_n   = 10240 ;		/* block length in bytes */

/* lists of pattern to use */
char**cmp_list ;
char**location_list;

/*-------------------------------------------------------------------------*/

/* scanning location list */
static int dds_location(){
	int eb,er,sb,sr;
	while ( *location_list != NULL ) {
		sscanf(*location_list,"%u:%u-%u:%u",&sb,&sr,&eb,&er);
		extract(sb,sr,eb,er);
		location_list++ ;
	}
}

/*-------------------------------------------------------------------------*/

/* reading archive location file */

#define LOCFILETEXT "first block number is %d\n" \
"block size is %d\n" "block length is %d\n"

static int dds_read_locfile(char*n){
	FILE *fp = fopen(n,"r") ;
	if ( fp == NULL ) {
		perror("dds2tar: can not open env file ");
		exit(1);
	}
	fscanf(fp,LOCFILETEXT,&tar_fb,&tar_bs,&tar_n);
	if ( tar_n != ( 512 * tar_bs )) {
		fprintf(stderr, "wrong block length, adjusted\n");
		tar_n = tar_bs * 512 ;
	}
	fclose(fp);
	if ( verbose ) fprintf(stderr,
		" location : ( %d , %d ) \n", tar_fb , tar_bs );
}

static int dds_print_locinfo(void){
	printf(LOCFILETEXT,dds_getpos(),tar_bs,tar_n);
}

/*-------------------------------------------------------------------------*/

int main(int argc,char**argv){

	int n = 1 ;
	int compressed_mode = T_MODE ;

	mode = MAKEINDEX ;

	/* scan arguments */
	while ( n < argc ) {
		if ( !strcmp(argv[n],"--help")){
			char*p=argv[0];
			fprintf(stderr,help_text,p,p,p,p,p);
			close(device);
			exit(0);
		} else
		if ( !strcmp(argv[n],"--version")){
			fprintf(stderr,"dds2tar: %s\n",version);
			close(device);
			exit(0);
		} else
		if ( !strcmp(argv[n],"-V")){
			fprintf(stderr,"dds2tar: %s\n",version);
			close(device);
			exit(0);
		} else
		if ( !strcmp(argv[n],"--verbose")){
			verbose = 1 ;
		} else
		if ( !strcmp(argv[n],"-v")){
			verbose = 1 ;
		} else
		if ( !strcmp(argv[n],"--hash-mode")){
			hash_mode = 1 ;
		} else
		if ( !strcmp(argv[n],"-L")){
			print_location = 1 ;
		} else
		if ( !strcmp(argv[n],"-B")){
			tar_bs =  atoi(argv[n+1]) ;
			tar_n = tar_bs * 512 ;
			n++ ;
		} else
		if ( !strcmp(argv[n],"-b")){
			tar_bs =  atoi(argv[n+1]) ;
			tar_n = tar_bs * 512 ;
			n++ ;
		} else
		if ( !strcmp(argv[n],"-e")){
			dds_read_locfile(argv[n+1]);
			n++ ;
		} else
		if ( !strcmp(argv[n],"-t")){
			index_file = argv[n+1] ;
			tar_index_file = 0 ;
			n++ ;
		} else
		if ( !strcmp(argv[n],"-R")){
			index_file = argv[n+1] ;
			tar_index_file = 1 ;
			n++ ;
		} else
		if ( !strcmp(argv[n],"-f")){
			device_name = argv[n+1] ;
			n++ ;
		} else
		if ( !strcmp(argv[n],"-a")){
			all = 1 ;
		} else
		if ( !strcmp(argv[n],"-z")){
			compressed_mode = C_MODE ;
		} else
#ifdef HPDAT
		if ( !strcmp(argv[n],"--ddscm-on")){
			device_change = 1 ;
			device_comp = DDSCM_ON ;
			dump_mode = 1 ;
		} else
		if ( !strcmp(argv[n],"--ddscm-off")){
			device_change = 1 ;
			device_comp = DDSCM_OFF ;
			dump_mode = 1 ;
		} else
		if ( !strcmp(argv[n],"--ddscm-query")){
			device_change = 1 ;
			device_comp = DDSCM_QUERY ;
			dump_mode = 1 ;
		} else
		if ( !strcmp(argv[n],"--ddscm-log")){
			device_change = 1 ;
			device_comp = DDSCM_LOG ;
			dump_mode = 1 ;
		} else
#endif
		if ( !strcmp(argv[n],"-x")){
			cmp_list = argv+n+1 ;
			mode = EXTRACT ;
			break ;
		} else
		if ( !strcmp(argv[n],"-p")){
			cmp_list = argv+n+1 ;
			mode =  PREFIX ;
			break ;
		} else
		if ( !strcmp(argv[n],"-g")){
			cmp_list = argv+n+1 ;
			mode =  GREP ;
			break ;
		} else
		if ( !strcmp(argv[n],"-l")){
			location_list = argv+n+1 ;
			mode = LOCATION ;
			break ;
		}
		n++ ;
	}
	{
		int open_mode = O_RDONLY ;	
		if ( dump_mode == 1 ) open_mode = O_WRONLY ;
		device = open_device(device_name,open_mode,verbose);
	}
#ifdef HPDAT
	if ( device_change == 1 ) {
		set_compression_mode(device,device_comp);
		exit(0);
	}
#endif

	if ( print_location != 0 ) {
		dds_print_locinfo();
		exit(0);
	}

	/* open table file */
	if ( mode == MAKEINDEX ) {
		index_fp = open_file(index_file,compressed_mode,"w");
	} else {
		index_fp = open_file(index_file,compressed_mode,"r");
	}

	if ( verbose ) fprintf(stderr,"selecting action ... \n");

	/* selecting the action */
	switch(mode){
		case MAKEINDEX: dds_index(); break;
		case LOCATION: dds_location(); break;
		case EXTRACT: dds_cmp(); break;
		case PREFIX: dds_cmp(); break;
		case GREP: dds_cmp(); break;
	}
	close(device);
}
